﻿#include "LoginWindow.h"
#include "ui_loginwindow.h"

#include <QGraphicsDropShadowEffect>
#include <QPainter>
#include <QLineEdit>
#include <QDebug>
#include <QRegExpValidator>
#include <QMessageBox>
#include <QMouseEvent>
#include <QCheckBox>
#include <QProcess>
#include <QButtonGroup>

#include "QAlerDialog.h"

#pragma execution_character_set("utf-8")

//#define LOCAL_DEBUG

LoginWindow::LoginWindow(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::LoginWindow)
{
    ui->setupUi(this);
    QGraphicsDropShadowEffect *shadowEffect = new QGraphicsDropShadowEffect(this);
    shadowEffect->setOffset(0, 0);
    shadowEffect->setColor(Qt::gray);
    shadowEffect->setBlurRadius(10);
    ui->frame->setGraphicsEffect(shadowEffect);
    this->setAttribute(Qt::WA_TranslucentBackground);

    // load settings
    loadSetting(true);

    ui->btnJoin->setEnabled(false);

    QFile file(":/stylesheet/style_buttom.qss");
    if (file.open(QFile::ReadOnly)) {
        this->setStyleSheet(tr(file.readAll()));
        file.close();
    }

    ui->txtRandom->installEventFilter(this);

//    QRegExp regx("[1-9][0-9]+$");
//    QValidator *validator = new QRegExpValidator(regx, ui->edtNumber);
//    ui->edtNumber->setValidator(validator);

#ifdef LOCAL_DEBUG
    ui->edtNumber->setText("9005524137");
    ui->tv_name->setText("hahahh");
    ui->tv_uid->setText("wwwe234");
#else
    ui->edtNumber->setText("");
    ui->tv_name->setText("");
    ui->tv_uid->setText("");
#endif

    qRegisterMetaType<SDKVideoFrame>("SDKVideoFrame");
    qRegisterMetaType<SDKCallInfo>("SDKCallInfo");
    qRegisterMetaType<SDKLoginInfo>("SDKLoginInfo");
    qRegisterMetaType<SDKLoginState>("SDKLoginState");
    qRegisterMetaType<ProxyNetAddress>("ProxyNetAddress");

    connect(NemoSdkAdaptor::getInstance(), &NemoSdkAdaptor::sigLoginResult, this, &LoginWindow::onLoginResult);
    connect(NemoSdkAdaptor::getInstance(), &NemoSdkAdaptor::sigKickOut, this, &LoginWindow::onKickOut);
    connect(NemoSdkAdaptor::getInstance(), &NemoSdkAdaptor::sigCallInvited, this, &LoginWindow::onCallInvited);
    connect(ui->tbtnSetting, &QToolButton::clicked, this, &LoginWindow::onSettingClicked);
    connect(ui->pb_logout, &QPushButton::clicked, this, &LoginWindow::onLogoutClicked);

    ui->stackedWidget->setCurrentIndex(0);
    connect(ui->bt_login, &QPushButton::clicked, this, &LoginWindow::onLoginClicked);
    connect(ui->cb_xy_acount_login, &QCheckBox::clicked, this, &LoginWindow::onLoginTypeClicked);

    updateUI();
}

LoginWindow::~LoginWindow()
{
    delete ui;
}

void LoginWindow::onLoginResult(SDKLoginState state, const QString& code, const SDKLoginInfo& info)
{
    if (state == SDKLoginState_LoggedIn) {
        if (code == SDKError_NoError) {
            ui->stackedWidget->setCurrentIndex(1);
            _displayName = info.displayName.str();
        } else {
            QAlerDialog *dialog = new QAlerDialog();
            dialog->showAlert(QAlerDialog::TYPE_TIPS, QString("登录失败, error: %1").arg(
                                  NemoSdkAdaptor::getInstance()->error_info(code)));
            delete dialog;
        }
    }
    if (state == SDKLoginState_LoggedOut) {
        if (code == SDKError_NoError) {
            ui->stackedWidget->setCurrentIndex(0);
            loadSetting();
            resetUnifiedLoginTestUi();
        } else {
            qDebug() << "-----> " << NemoSdkAdaptor::getInstance()->error_info(code);
            QAlerDialog *dialog = new QAlerDialog();
            dialog->showAlert(QAlerDialog::TYPE_TIPS, QString("操作失败, error: %1").arg(
                                  NemoSdkAdaptor::getInstance()->error_info(code)));
            delete dialog;
        }
    }
}

void LoginWindow::onCallInvited(const SDKCallInfo& info)
{
    this->hide();
    meetingWindow = new MeetingWindow();
    meetingWindow->show();
    meetingWindow->onMeetingInvited(_displayName, info, !ui->cbCamera->isChecked(), !ui->cbMic->isChecked());
    connect(meetingWindow, &MeetingWindow::sigMeetingFinished, this, &LoginWindow::onCallFinished);
}

void LoginWindow::onConnectionChanged(bool state)
{
    qDebug() << "onConnectionChanged : " << state;
    ui->btnJoin->setEnabled(state);
}

void LoginWindow::onKickOut(const QString& code)
{
    qDebug() << "onKickOut : " << code;
    NemoSdkAdaptor::getInstance()->logout();
    QAlerDialog *dialog = new QAlerDialog();
    dialog->showAlert(QAlerDialog::TYPE_TIPS, NemoSdkAdaptor::getInstance()->error_info(code));
    delete dialog;
}


void LoginWindow::mousePressEvent(QMouseEvent *e)
{
    last = e->globalPos();
}

void LoginWindow::mouseMoveEvent(QMouseEvent *e)
{
    int dx = e->globalX() - last.x();
    int dy = e->globalY() - last.y();
    last = e->globalPos();
    move(x()+dx, y()+dy);
}

void LoginWindow::mouseReleaseEvent(QMouseEvent *e)
{
    int dx = e->globalX() - last.x();
    int dy = e->globalY() - last.y();
    move(x()+dx, y()+dy);
}

void LoginWindow::onCallFinished(QString reason)
{
    meetingWindow->close();
    meetingWindow->deleteLater();
    this->show();
}

void LoginWindow::on_btnJoin_clicked()
{
    if(NemoSdkAdaptor::getInstance()->isConnected()) {
        this->hide();
        meetingWindow = new MeetingWindow();
        meetingWindow->show();
        QString loginDisplayName = ui->tv_name->text();
        meetingWindow->makeCall(ui->edtNumber->text(), loginDisplayName.isEmpty() ? _displayName : loginDisplayName, "",
                                !ui->cbCamera->isChecked(), !ui->cbMic->isChecked(), true, ui->tv_meeting_id->text());
        connect(meetingWindow, &MeetingWindow::sigMeetingFinished, this, &LoginWindow::onCallFinished);
    } else {
        QAlerDialog *dialog = new QAlerDialog();
        dialog->showAlert(QAlerDialog::TYPE_TIPS, ("Network unavailable"));
        delete dialog;
    }
}

void LoginWindow::on_tbtnClose_clicked()
{
    if (!NemoSdkAdaptor::getInstance()->isSdkClosed()){
        NemoSdkAdaptor::getInstance()->shutdown();
    }
    this->close();
}

void LoginWindow::on_tbtnMinise_clicked()
{
    this->showMinimized();
}

void LoginWindow::on_edtNumber_textChanged(const QString &arg1)
{
    ui->btnJoin->setEnabled(arg1.length() > 0);
}

void LoginWindow::onLoginClicked()
{
    QString displayName = ui->tv_name->text();
    QString uuid = ui->tv_uid->text();
    qDebug() << "onLoginClicked: " << extId;
#ifdef UNIFIED_LOGIN_TEST
    QString settingConfig = QCoreApplication::applicationDirPath() + "/config.ini";
    QSettings configs(settingConfig, QSettings::IniFormat);
    configs.beginGroup("serverConfig");
    QString extId = configs.value("extId").toString();
    if (ui->cb_unified_login_test->isChecked()) {
        if (ui->cb_loginXYAccount->isChecked()) {
            if (displayName.isEmpty() || uuid.isEmpty()) {
                QAlerDialog *dialog = new QAlerDialog();
                dialog->showAlert(QAlerDialog::TYPE_TIPS, ("入会名称/账号不能为空"));
                delete dialog;
                return;
            }
            QStringList accountList = displayName.split("-");
            if (accountList.size() == 2) {
                NemoSdkAdaptor::getInstance()->loginXYAccount(extId.toStdString(), accountList.at(0).toStdString(), accountList.at(1).toStdString(), uuid.toStdString());
            } else {
                NemoSdkAdaptor::getInstance()->loginXYAccount(extId.toStdString(), "+86", displayName.toStdString(), uuid.toStdString());
            }
        } else if (ui->cb_loginExtToken->isChecked()) {
            if (displayName.isEmpty()) {
                QAlerDialog *dialog = new QAlerDialog();
                dialog->showAlert(QAlerDialog::TYPE_TIPS, ("token不能为空"));
                delete dialog;
                return;
            }
            NemoSdkAdaptor::getInstance()->loginExtToken(extId.toStdString(), displayName.toStdString());
        } else if (ui->cb_loginExtUserId->isChecked()) {
            QString authCode = ui->tv_authCode->text();
            if (displayName.isEmpty() || uuid.isEmpty() || authCode.isEmpty()) {
                QAlerDialog *dialog = new QAlerDialog();
                dialog->showAlert(QAlerDialog::TYPE_TIPS, ("authCode/extuid不能为空"));
                delete dialog;
                return;
            }
            NemoSdkAdaptor::getInstance()->loginExtUserId(extId.toStdString(), displayName.toStdString(), uuid.toStdString(), authCode.toStdString());
        }
    } else {
        if (ui->cb_xy_acount_login->isChecked()) {
            if (displayName.isEmpty() || uuid.isEmpty()) {
                QAlerDialog *dialog = new QAlerDialog();
                dialog->showAlert(QAlerDialog::TYPE_TIPS, ("入会名称/账号不能为空"));
                delete dialog;
                return;
            }
            NemoSdkAdaptor::getInstance()->login(displayName.toStdString(), uuid.toStdString());
        } else {
            if (displayName.isEmpty() || uuid.isEmpty()) {
                QAlerDialog *dialog = new QAlerDialog();
                dialog->showAlert(QAlerDialog::TYPE_TIPS, ("入会名称/账号不能为空"));
                delete dialog;
                return;
            }
            QString settingConfig = QCoreApplication::applicationDirPath() + "/config.ini";
            QSettings configs(settingConfig, QSettings::IniFormat);
            configs.beginGroup("serverConfig");
            QString extId = configs.value("extId").toString();
            NemoSdkAdaptor::getInstance()->loginWithExtId(extId.toStdString(), uuid.toStdString(), displayName.toStdString());
        }
    }
#else
    if (displayName.isEmpty() || uuid.isEmpty()) {
        QAlerDialog *dialog = new QAlerDialog();
        dialog->showAlert(QAlerDialog::TYPE_TIPS, ("入会名称/账号不能为空"));
        delete dialog;
        return;
    }
    if (ui->cb_xy_acount_login->isChecked()) {
        NemoSdkAdaptor::getInstance()->login(displayName.toStdString(), uuid.toStdString());
    } else {
        QString settingConfig = QCoreApplication::applicationDirPath() + "/config.ini";
        QSettings configs(settingConfig, QSettings::IniFormat);
        configs.beginGroup("serverConfig");
        QString extId = configs.value("extId").toString();
        NemoSdkAdaptor::getInstance()->loginWithExtId(extId.toStdString(), uuid.toStdString(), displayName.toStdString());
    }
    // 免密快速登录
    //NemoSdkAdaptor::getInstance()->quickLogin(enterpriseId.toStdString().c_str(), displayName.toStdString().c_str());
#endif
}

void LoginWindow::onSettingClicked()
{
    XYSettings* settings = new XYSettings();
    settings->setAttribute(Qt::WA_DeleteOnClose);
    settings->show();
    connect(settings, &XYSettings::sigServerChanged, this, &LoginWindow::onServerChanged);

    connect(settings, &XYSettings::sigProxyChange, this, &LoginWindow::onProxyChanged);
}

/**
 * server changed need re-login
 *
 * @brief LoginWindow::onServerChanged
 * @param host
 */
void LoginWindow::onServerChanged()
{
    qDebug() << __func__;
    if (ui->stackedWidget->currentIndex() == 1) {
        // 登录状态下需要登出
        NemoSdkAdaptor::getInstance()->logout();
        ui->stackedWidget->setCurrentIndex(0); // test remove
        loadSetting();
    } else {
        loadSetting();
    }
}

/**
 * @brief LoginWindow::onProxyChanged re-init sdk
 * @param type
 * @param proxy
 */
void LoginWindow::onProxyChanged(std::map<ProxyType, ProxyNetAddress> proxys)
{
    if (ui->stackedWidget->currentIndex() == 1) {
        NemoSdkAdaptor::getInstance()->logout();
        ui->stackedWidget->setCurrentIndex(0);
    }

    QString program = QApplication::applicationFilePath();
    QStringList arguments = QApplication::arguments();

    qApp->quit();
    QProcess::startDetached(program, arguments);
}

void LoginWindow::onLogoutClicked()
{
    NemoSdkAdaptor::getInstance()->logout();
}

void LoginWindow::onLoginTypeClicked(bool checked) {
    if (checked) {
        ui->tv_name->setPlaceholderText(tr("手机号/邮箱"));
        ui->tv_uid->setPlaceholderText(tr("密码"));
    } else {
        ui->tv_name->setPlaceholderText(tr("输入显示名称"));
        ui->tv_uid->setPlaceholderText(tr("输入用户ID"));
    }
}

void LoginWindow::loadSetting(bool firstInit)
{
    QString settingConfig = QCoreApplication::applicationDirPath() + "/config.ini";
    qDebug() << "settingConfig: " << settingConfig;
    QSettings configs(settingConfig, QSettings::IniFormat);
    configs.beginGroup("serverConfig");
    QString host = configs.value("host").toString();
    QString sysType = configs.value("sysType").toString();
    QString clientId = configs.value("clientId").toString();
    QString clientSecret = configs.value("clientSecret").toString();
    QString extId = configs.value("extId").toString();
    if (host.isEmpty()) {
        // set default config
        configs.setValue("host", "sdkapi.xylink.com");
        host = "sdkapi.xylink.com";
    }
    if (sysType.isEmpty()) {
        // set default sysType
        configs.setValue("sysType", "c++sdk");
    }
    if (firstInit) {
        if (clientId.isEmpty()) {
            // set default clientId
            configs.setValue("clientId", "l9Asj75NryPJaK3sshFFksuo");
            clientId = "l9Asj75NryPJaK3sshFFksuo";
        }
    }

    if (clientSecret.isEmpty()) {
        // set default clientSecret
        configs.setValue("clientSecret", "tUvFBtsIiST7oFn31Tap781qC5kFcfXH");
        clientSecret = "tUvFBtsIiST7oFn31Tap781qC5kFcfXH";
    }
    if (extId.isEmpty()) {
        // set default extId
        configs.setValue("extId", "0142901e3d83e0a1e225ef92b8663fcaebda7242");
        extId = "0142901e3d83e0a1e225ef92b8663fcaebda7242";
    }
    qDebug() << "===>" << host << clientId << clientSecret;
    NemoSdkAdaptor::getInstance()->init(host, extId, clientId, clientSecret);

    QString httpProxyHost = configs.value(HTTP_PROXY_HOST).toString();
    QString httpProxyPort = configs.value(HTTP_PROXY_PORT).toString();
    if (!httpProxyHost.isEmpty() && !httpProxyPort.isEmpty())
    {
        ProxyNetAddress proxy;
        proxy.hostAddress = httpProxyHost.toStdString().c_str();
        proxy.port = httpProxyPort.toInt(0);
        NemoSdkAdaptor::getInstance()->setProxy(ProxyType::HTTP, proxy);
    }

    QString udpProxyHost = configs.value(UDP_PROXY_HOST).toString();
    QString udpProxyPort = configs.value(UDP_PROXY_PORT).toString();
    if (!udpProxyHost.isEmpty() && !udpProxyPort.isEmpty())
    {
        ProxyNetAddress proxy;
        proxy.hostAddress = udpProxyHost.toStdString().c_str();
        proxy.port = udpProxyPort.toInt(0);
        NemoSdkAdaptor::getInstance()->setProxy(ProxyType::UDP, proxy);
    }

    QString sock5ProxyHost = configs.value(SOCK5_PROXY_HOST).toString();
    QString sock5ProxyPort = configs.value(SOCK5_PROXY_PORT).toString();
    QString sock5ProxyName = configs.value(SOCK5_PROXY_NAME).toString();
    QString sock5ProxyPwd = configs.value(SOCK5_PROXY_PWD).toString();
    if (!sock5ProxyHost.isEmpty() && !sock5ProxyPort.isEmpty())
    {
        ProxyNetAddress proxy;
        proxy.hostAddress = sock5ProxyHost.toStdString().c_str();
        proxy.port = sock5ProxyPort.toInt(0);
        proxy.username = sock5ProxyName.toStdString().c_str();
        proxy.password = sock5ProxyPwd.toStdString().c_str();
        NemoSdkAdaptor::getInstance()->setProxy(ProxyType::SOCK5, proxy);
    }
}


void LoginWindow::updateUI()
{
#ifdef UNIFIED_LOGIN_TEST
    ui->cb_unified_login_test->setChecked(false);
    ui->cb_xy_acount_login->setChecked(false);
    ui->tv_meeting_id->setVisible(false);
    ui->text_tips->setVisible(true);
    QButtonGroup* bg = new QButtonGroup;
    bg->addButton(ui->cb_loginXYAccount);
    bg->addButton(ui->cb_loginExtToken);
    bg->addButton(ui->cb_loginExtUserId);
    bg->setExclusive(true);
    if (ui->cb_unified_login_test->isChecked()) {
        ui->cb_xy_acount_login->setVisible(false);
    } else {
        ui->cb_loginXYAccount->setVisible(false);
        ui->cb_loginExtToken->setVisible(false);
        ui->cb_loginExtUserId->setVisible(false);
        ui->tv_authCode->setVisible(false);
    }
    connect(ui->cb_loginXYAccount, &QRadioButton::clicked, this, [=](bool checked) {
        if (checked) {
            ui->tv_name->setText("");
            ui->tv_uid->setText("");
            ui->tv_name->setPlaceholderText(tr("手机号/邮箱"));
            ui->tv_uid->setVisible(true);
            ui->tv_uid->setPlaceholderText(tr("密码"));
            ui->tv_authCode->setVisible(false);
        }
    });
    connect(ui->cb_loginExtToken, &QRadioButton::clicked, this, [=](bool checked) {
        if (checked) {
            ui->tv_name->setText("");
            ui->tv_name->setPlaceholderText(tr("token"));
            ui->tv_uid->setVisible(false);
            ui->tv_authCode->setVisible(false);
        }
    });
    connect(ui->cb_loginExtUserId, &QRadioButton::clicked, this, [=](bool checked) {
        if (checked) {
            ui->tv_authCode->setText("");
            ui->tv_name->setText("");
            ui->tv_uid->setText("");
            ui->tv_uid->setVisible(true);
            ui->tv_name->setPlaceholderText(tr("输入显示名称"));
            ui->tv_uid->setPlaceholderText(tr("输入用户ID"));
            ui->tv_authCode->setVisible(true);
        }
    });
    connect(ui->cb_unified_login_test, &QRadioButton::clicked, this, [=](bool checked) {
        if (checked) {
            ui->tv_name->setText("");
            ui->tv_uid->setText("");
            ui->cb_loginXYAccount->setChecked(true);
            ui->tv_name->setPlaceholderText(tr("手机号/邮箱"));
            ui->tv_uid->setVisible(true);
            ui->tv_uid->setPlaceholderText(tr("密码"));
        } else {
            ui->tv_name->setText("");
            ui->tv_uid->setText("");
            ui->tv_authCode->setVisible(false);
            if (ui->cb_xy_acount_login->isChecked()) {
                ui->tv_uid->setVisible(true);
                ui->tv_name->setPlaceholderText(tr("手机号/邮箱"));
                ui->tv_uid->setPlaceholderText(tr("密码"));
            } else {
                ui->tv_uid->setVisible(true);
                ui->tv_name->setPlaceholderText(tr("输入显示名称"));
                ui->tv_uid->setPlaceholderText(tr("输入用户ID"));
            }
        }
        ui->cb_loginXYAccount->setVisible(checked);
        ui->cb_loginExtToken->setVisible(checked);
        ui->cb_loginExtUserId->setVisible(checked);
        ui->cb_xy_acount_login->setVisible(!checked);
    });
#else
    ui->unified_bt_holder->setVisible(false);
    ui->tv_authCode->setVisible(false);
    ui->cb_unified_login_test->setVisible(false);
    ui->text_tips->setVisible(false);
    ui->cb_xy_acount_login->setVisible(true);
    QButtonGroup* bg = new QButtonGroup;
    bg->addButton(ui->cb_xy_acount_login);
    bg->setExclusive(false);
#endif
}

void LoginWindow::resetUnifiedLoginTestUi()
{
#ifdef UNIFIED_LOGIN_TEST
    ui->tv_name->setText("");
    ui->tv_uid->setText("");
    ui->tv_authCode->setText("");
    ui->edtNumber->setText("");
#endif
}

bool LoginWindow::eventFilter(QObject *obj, QEvent *event)
{
    if(obj == ui->txtRandom && event->type() == QEvent::MouseButtonPress) {
        QAlerDialog *dialog = new QAlerDialog();
        dialog->showAlert(QAlerDialog::TYPE_TIPS, ("暂未开发"));
        delete dialog;
        return true;
    }
    return QWidget::eventFilter(obj, event);
}
