﻿#include "NemoSdkAdaptor.h"
#include <QLibrary>
#include <QMessageBox>
#include <QDebug>
#include <iostream>
#include <fstream>
#include <stdio.h>
#include <string>
#include <stdlib.h>
#include <QDir>
#include <QProcessEnvironment>

typedef NemoSDK::INemoSDK* (*GET_NEMOSDK_INSTANCE)();

NemoSdkAdaptor *NemoSdkAdaptor::getInstance()
{
    static NemoSdkAdaptor instance;
    return &instance;
}

QString NemoSdkAdaptor::error_info(const QString &code)
{
    return jsonObj.value(code).toString();
}

void NemoSdkAdaptor::init(const QString& host, const QString& extId, const QString& clientId,
                                    const QString& clientSecret)
{
    qDebug() << __func__;
#ifdef WIN32
    QLibrary mylib("NemoSDK.dll");
#else
    QLibrary mylib("librtcsdk.so");
#endif // WIN32


    if(mylib.load())
    {
        GET_NEMOSDK_INSTANCE getInsFunc=(GET_NEMOSDK_INSTANCE)mylib.resolve("getNemoSdkInstance");
        if (getInsFunc)
        {
            _sdk=getInsFunc();
        }
        else
        {
            QMessageBox::information(NULL,"NO","Linke to Function is not OK!!!!");
        }
    }
    else
    {
        QMessageBox::information(NULL,"NO","DLL is not loaded: " + mylib.errorString());
        qDebug() << "load error : " << mylib.errorString();
    }

    if (_sdk != nullptr) {
#ifndef Q_OS_LINUX
        QString dir_str = "./log";
        QString mediaDump = "./mediaDump";
        QDir dir;
        if (!dir.exists(dir_str))
        {
            bool res = dir.mkpath(dir_str);
            qDebug() << "create log folder success" << res;
        }

        if (!dir.exists(mediaDump))
        {
            bool res = dir.mkpath(mediaDump);
            qDebug() << "create mediaDump folder success" << res;
        }
#endif

#ifdef Q_OS_LINUX
        std::string logFolder = "/opt/apps/com.xylink.sdkdemo/files/xylink/log";
#else
        std::string logFolder = "./log";
#endif
        std::string sCloudServer = host.isEmpty() ? "" : host.toStdString();
        std::string openapiToken = "a8208f6495ba1d468343cc63c13c5202837b5ba8614bdc699f46e91b4df09fd0";
        std::string openapiServer = "txdev-sdkapi.xylink.com";
        std::string sCliengId = clientId.isEmpty() ? "" : clientId.toStdString();
        std::string sSlientSecret = clientSecret.isEmpty() ? "" : clientSecret.toStdString();

        SDKConfigs configs;
        configs.logFolder = logFolder.c_str();
        configs.cloudServer = sCloudServer.c_str();
        configs.openapiToken = openapiToken.c_str();
        configs.openapiServer = openapiServer.c_str();
        configs.clientId = sCliengId.c_str();
        configs.mediaDumpPath = logFolder.c_str();
        configs.clientSecret = sSlientSecret.c_str();
        configs.sysType = SYS_TYPE_CPP_SDK;
        _sdk->startup(this, configs);
        getGPUInfo();
        sdkClosed = false;
    }
}

void NemoSdkAdaptor::makeCall(const std::string& number, const std::string& meetingPwd, SDKCallMode mode,
                              const std::string& displayName, const QString& meetingId)
{
    log(QString("makeCall : %1").arg(number.c_str()));
    std::string stdMeetingId = meetingId.toStdString();
    _sdk->makeCall(number.c_str(), meetingPwd.c_str(), mode, displayName.c_str(), false, false, false, stdMeetingId.c_str());
}

void NemoSdkAdaptor::dropCall()
{
    _sdk->hangup();
}

bool NemoSdkAdaptor::isSdkClosed() const
{
    return sdkClosed;
}

bool NemoSdkAdaptor::isLogedIn() const
{
    return _isLogedIn;
}

const XYArray<SDKMonitorInfo>& NemoSdkAdaptor::getMonitorList()
{
    return _sdk->getMonitorList();
}

void NemoSdkAdaptor::enableLocalFD(SDKFaceType faceType, bool enable)
{
    _sdk->enableFaceDetectMode(faceType, enable);
}

void NemoSdkAdaptor::quickLogin(const char *extID, const char *account)
{
    _sdk->loginEnterpriseAccount(extID, account);
}

void NemoSdkAdaptor::getFaceInfo(const XYArray<int32_t> &faceId)
{
    _sdk->getFaceInfo(faceId);
}

void NemoSdkAdaptor::enableEletronicBadge(bool enable)
{
    _sdk->broadcastEletronicBadge(enable);
}

void NemoSdkAdaptor::enableAudioDump()
{
    _sdk->enableAudioDump();
}

void NemoSdkAdaptor::setVirtualBgImage(const char *imgPath)
{
    _sdk->setVirtualBgImage(imgPath);
}

void NemoSdkAdaptor::setVirtualBgMode(SDKVirtualBgMode mode)
{
    _sdk->setVirtualBgMode(mode);
}

void NemoSdkAdaptor::setVideoBeautyEffect(SDKBeautyStyle style, int effectLevel)
{
    _sdk->setVideoBeautyEffect(style, effectLevel);
}

void NemoSdkAdaptor::setVideoFilterEffect(SDKFilterStyle style, int effectLevel)
{
    _sdk->setVideoFilterEffect(style, effectLevel);
}

void NemoSdkAdaptor::logUpload(const std::string& comments)
{
    _sdk->logUpload(comments.c_str());
}

void NemoSdkAdaptor::getGPUInfo()
{
    _sdk->getGPUInfo();
}

void NemoSdkAdaptor::switchCallMode(bool audioMode, bool audioMute, bool videoMute)
{
    _sdk->switchCallMode(audioMode, audioMute, videoMute);
}

void NemoSdkAdaptor::enableAEC(bool enable)
{
    _sdk->enableAECMode(enable);
}

bool NemoSdkAdaptor::isRecordEnable() const {
    return recordEnable;
}

void NemoSdkAdaptor::setRecordEnable(bool enable) {
    recordEnable = enable;
}

bool NemoSdkAdaptor::isRecordStartBySelf() const
{
    return _isRecordStartBySelf;
}

void NemoSdkAdaptor::startRecord()
{
    qDebug() << __func__;
    _sdk->startCloudRecord();
    _isRecordStartBySelf = true;
}

void NemoSdkAdaptor::stopRecord()
{
    qDebug() << __func__;
    _sdk->stopCloudRecord();
    _isRecordStartBySelf = false;
}

QString NemoSdkAdaptor::getConfMgmtUrl(const char *confNum)
{
    return QString(_sdk->getConfMgmtUrl(confNum).str());
}

void NemoSdkAdaptor::releaseAudioCapSource()
{
    _sdk->releaseAudioFocus();
}

void NemoSdkAdaptor::enablePulseHelper(bool enable)
{
    _sdk->enablePulseHelper(enable);
}

void NemoSdkAdaptor::putContentData(void *argb, size_t bufLen, int width, int height, int stride)
{
     _sdk->putContentData(argb, bufLen, width, height, stride);
}

void NemoSdkAdaptor::setSubtitleLanguage(SDKLanguageType type)
{
    _sdk->setSubtitleLanguage(type);
}

void NemoSdkAdaptor::checkAiCaptionSupport()
{
    _sdk->checkAiCaptionSupport();
}

void NemoSdkAdaptor::setProxy(ProxyType type, const ProxyNetAddress &proxy)
{
    qDebug() << __func__ << type <<  proxy.hostAddress.str() << proxy.port;
    _sdk->setProxy(type, proxy);
}

NemoSdkAdaptor::NemoSdkAdaptor()
{
    QFile josnFile(":/raw/ErrorCode.json");
    if (josnFile.open(QIODevice::ReadOnly)) {
        QByteArray jsonArray = josnFile.readAll();
        QJsonParseError jsonError;
        jsonObj = QJsonDocument::fromJson(jsonArray, &jsonError).object();
        if (jsonError.error != QJsonParseError::NoError)
            qDebug() << "jsonparse json parse got an error";
    }
    josnFile.close();
#ifdef Q_OS_WIN
    _isWayland = false;
#else
    auto e = QProcessEnvironment::systemEnvironment();
    QString XDG_SESSION_TYPE = e.value(QStringLiteral("XDG_SESSION_TYPE"));
    QString WAYLAND_DISPLAY = e.value(QStringLiteral("WAYLAND_DISPLAY"));
    _isWayland = XDG_SESSION_TYPE == QLatin1String("wayland") || WAYLAND_DISPLAY.contains(QLatin1String("wayland"), Qt::CaseInsensitive);
#endif

#ifdef WAYLAND
    if (_isWayland)
    {
        _waylandCapture = std::unique_ptr<WaylandCapture>(new WaylandCapture(this));
    }
#endif
}

NemoSdkAdaptor::~NemoSdkAdaptor()
{
    qDebug() << "~NemoSdkAdaptor()";
    if (_sdk)
    {
        _sdk->shutdown();
    }
}

void NemoSdkAdaptor::shutdown()
{
    if (!sdkClosed) {
        _sdk->shutdown();
    }
    sdkClosed = true;
    qDebug() << "shutdown: ";
}

void NemoSdkAdaptor::setChairman(const std::string& name)
{
    //_sdk->setChairman(name.c_str());
    log(QString("setChairman : %1").arg(name.c_str()));
    qDebug() << "setChairman:" << name.c_str();
}

void NemoSdkAdaptor::setMicMute(bool mute)
{
    log(QString("setMicMute : %1").arg(mute));
    _sdk->setMicMute(mute);
    micMuted = mute;
}

void NemoSdkAdaptor::setVideoMute(bool mute)
{
    log(QString("setVideoMute : %1").arg(mute));
    _sdk->setVideoMute(mute);
    videoMuted = mute;
}

void NemoSdkAdaptor::setAudioPeakMeterEnabled(bool enable)
{
    _sdk->setAudioPeakMeterEnabled(enable);
}

void NemoSdkAdaptor::startCamera()
{
    log(QString("startCamera"));
    _sdk->startCamera();
    cameraClosed = false;
}
void NemoSdkAdaptor::stopCamera()
{
    log(QString("stopCamera"));
    qDebug() << __func__;
    _sdk->stopCamera();
    cameraClosed = true;
}

void NemoSdkAdaptor::login(const std::string& name, const std::string& pwd)
{
    _sdk->loginXYLinkAccount(name.c_str(), pwd.c_str());
}

void NemoSdkAdaptor::loginXYAccount(const std::string &extId, const std::string& countryCode, const std::string &account, const std::string &pwd)
{

}

void NemoSdkAdaptor::loginExtToken(const std::string &extId, const std::string &token)
{

}

void NemoSdkAdaptor::loginExtUserId(const std::string &extId, const std::string &displayName, const std::string &extUID, const std::string &authCode)
{

}

void NemoSdkAdaptor::logout()
{
    _sdk->logout();
}

void NemoSdkAdaptor::acceptCall()
{
    _sdk->acceptCall();
}

void NemoSdkAdaptor::rejectCall()
{
    _sdk->rejectCall();
}

void NemoSdkAdaptor::loginWithExtId(const std::string& extID, const std::string& extUserId, const std::string& displayName)
{
    _sdk->loginExternalAccount(extID.c_str(), extUserId.c_str(), displayName.c_str());
}

std::vector<SDKMediaDevInfo> NemoSdkAdaptor::getDevList(SDKMediaDevType t)
{
    std::vector<SDKMediaDevInfo> infosVec;
    XYArray<SDKMediaDevInfo> infos = _sdk->getDevList(t);
    infosVec.reserve(infos.length());
    for (int i = 0; i < infos.length(); i++) {
        infosVec.push_back(std::move(infos[i]));
    }
    return infosVec;
}

const SDKStatistics& NemoSdkAdaptor::getStatistics()
{
    return _sdk->getStatistics();
}

void NemoSdkAdaptor::chooseDev(SDKMediaDevType t, const char *devId)
{
    _sdk->chooseDev(t, devId);
}

void NemoSdkAdaptor::setVideoProfile(const SDKVideoProfile& profile)
{
    _sdk->setVideoProfile(profile);
}

void NemoSdkAdaptor::setSpeakerVolume(int volume)
{
    _sdk->setSpeakerVolume(volume);
}

int NemoSdkAdaptor::getSpeakerVolume()
{
    qDebug()<<  __func__ << _sdk->getSpeakerVolume();
    return _sdk->getSpeakerVolume();
}

void NemoSdkAdaptor::startPlayWavFile(const char *wavfile, int loopTimes)
{
    _sdk->startPlayWavFile(wavfile, loopTimes);
}

void NemoSdkAdaptor::stopPlayWavfile()
{
    _sdk->stopPlayWavfile();
}

int NemoSdkAdaptor::getSpeakerPeakMeter()
{
    return _sdk->getSpeakerPeakMeter();
}

void NemoSdkAdaptor::enableListnerTest(bool enable)
{
    _sdk->enableMicTest(enable);
}

int NemoSdkAdaptor::getMicPeakMeter()
{
    return _sdk->getMicPeakMeter();
}

void NemoSdkAdaptor::setMicVolume(int value)
{
    _sdk->setMicVolume(value);
}

void NemoSdkAdaptor::startWhiteboard()
{
    ///_sdk->startWhiteboard();
}

void NemoSdkAdaptor::stopWhiteboard()
{
    ///_sdk->stopWhiteboard();
}

void NemoSdkAdaptor::sendWhiteboardMessage(const char * msg)
{
    ///_sdk->sendWhiteboardMessage(msg);
}

const XYArray<SDKAudioMeterInfo>& NemoSdkAdaptor::getAudioPeakMeters()
{
    return _sdk->getAudioPeakMeters();
}

bool NemoSdkAdaptor::isCameraClosed()const
{
    return cameraClosed;
}

bool NemoSdkAdaptor::isVideoMuted()const
{
    return videoMuted;
}

bool NemoSdkAdaptor::isMicMuted()const
{
    return micMuted;
}

bool NemoSdkAdaptor::isConnected() const
{
    return connected;
}

bool NemoSdkAdaptor::isContentSharing()const
{
    return contentSharing;
}

void NemoSdkAdaptor::startContentSharing(const QString& monitor, const SDKContentInfo& contentInfo)
{
#ifdef WAYLAND
    if (_isWayland)
    {
        _waylandCapture->startContentCapture();
    }
#endif
    qDebug() << __FUNCTION__ << contentInfo.region.x << contentInfo.region.y << contentInfo.region.width << contentInfo.region.height;
    _sdk->startContentSharing(monitor.toStdString().c_str(), contentInfo);
}

void NemoSdkAdaptor::updateContentCaptureRegion(int x, int y, int width, int height)
{
    _sdk->updateContentCaptureRegion(x, y, width, height);
}

void NemoSdkAdaptor::stopContentSharing()
{
    _sdk->stopContentSharing();
#ifdef WAYLAND
    if (_isWayland)
    {
        _waylandCapture->stopContentCapture();
    }
#endif
}

void NemoSdkAdaptor::suspendContentShare()
{
    _sdk->suspendContentShare();
#ifdef WAYLAND
    if (_isWayland)
    {
        _waylandCapture->pauseContentCapture();
    }
#endif
}

void  NemoSdkAdaptor::restartContentShare()
{
    _sdk->restartContentShare();
#ifdef WAYLAND
    if (_isWayland)
    {
        _waylandCapture->restartContentCapture();
    }
#endif
}

void NemoSdkAdaptor::log(const QString& info)
{
    _sdk->dumpInfo(info.toStdString().c_str());
}

void NemoSdkAdaptor::sendSpeakingRequest()
{
    _sdk->sendSpeakingRequest();
}

void NemoSdkAdaptor::sendSpeakingEnd()
{
    _sdk->sendSpeakingEnd();
}

void NemoSdkAdaptor::cancelSpeakingRequest()
{
    _sdk->cancelSpeakingRequest();
}

void NemoSdkAdaptor::convertYUV2ARGB(const uint8_t* data, uint8_t* dst_argb, size_t bufLen, int width, int height, int rotate, bool flip)
{
    _sdk->convertYUV2ARGB(data, dst_argb, bufLen, width, height, rotate, flip);
}

void NemoSdkAdaptor::reqAutoLayout(int num, bool hasContent, const std::string& chairmanUri, MeetingLayout::LayoutMode layoutMode,
                                   int pageIndex, bool uiShowLocalWhenPageMode, const std::vector<QString>& calluris)
{
    SDKLayoutReq req;
    req.pageIndex = pageIndex;
    req.maxViewCountPerPage = num;
    req.uiShowLocalWhenPageMode = uiShowLocalWhenPageMode;
    req.reqList.init(num);
    if (chairmanUri.length() > 0) {
        for(int i = 0; i < num; i++)
        {
            SDKViewReqInfo info;
            if(i == 0)
            {
                info.isContent = hasContent;
                info.resolution = VideoResolution720P;
                info.quality = VideoQualityHigh;
//                info.reqFaceDetect = true;
                if (!hasContent) {
                    info.callUri = chairmanUri.c_str();
                }
                req.reqList[i] = info;
            } else {
                info.isContent = false;
//                info.reqFaceDetect = false;
                info.resolution = VideoResolution180P;
                info.quality = VideoQualityNormal;
                req.reqList[i] = info;
                if (hasContent) {
                    info.callUri = chairmanUri.c_str();
                }
            }
        }
    } else {
        if (layoutMode == MeetingLayout::ContentAndAs) {
            for(int i = 0; i < num; i++) {
                SDKViewReqInfo info;
                info.isContent = i == 0 ? hasContent : false;
                info.resolution = VideoResolution360P;
                info.quality = VideoQualityNormal;
                req.reqList[i] = info;
            }
        } else if (layoutMode == MeetingLayout::Gallery || layoutMode == MeetingLayout::ContentShare) {
            for(int i = 0; i < num; i++) {
                SDKViewReqInfo info;
                info.isContent = false;
                info.resolution = VideoResolution180P;
                info.quality = VideoQualityNormal;
                req.reqList[i] = info;
            }
        } else {
            for(int i = 0; i < num; i++) {
                SDKViewReqInfo info;
                if (calluris.size() > 0 && i < calluris.size()) {
                    info.callUri = calluris[i].toStdString().c_str();
                }
                info.isContent = i == 0 ? hasContent : false;
//                info.reqFaceDetect = i == 0 ? true : false;
                info.resolution = i == 0 ? (hasContent ? VideoResolution1080P : VideoResolution720P) : VideoResolution180P;
                info.quality = i == 0 ? VideoQualityHigh : VideoQualityNormal;
                req.reqList[i] = info;
            }
        }
    }
    _sdk->requestLayout(req);
    qDebug() << "reqAutoLayout, num : " << num << ", hasContent : " << hasContent << ", chirman : " << chairmanUri.c_str();
}

void NemoSdkAdaptor::onError(const XYString& code)
{
    log(QString("onSDKError : %1").arg(code.str()));
    qDebug() << "onSDKError:" << code.str();
}

void NemoSdkAdaptor::onKickOut(const XYString& code)
{
    log(QString("onKickOut : %1").arg(code.str()));
    qDebug() << "onKickOut:" << code.str();
    sigKickOut(QString(code.str()));
}

void NemoSdkAdaptor::onLoginStateChanged(SDKLoginState state, const XYString& code, const SDKLoginInfo& info/*only valid when state==LoggedIn*/)
{
    QString errCode(code.str());
    _isLogedIn = (errCode == SDKError_NoError && state == SDKLoginState_LoggedIn);
    qDebug() << "qt onLoginState: state=" << state << ", error=" << errCode << ", uri=" << info.callUri.str() << ", uid : " << info.userId;
    sigLoginResult(state, errCode, info);
    const char*ch = info.callUri.str();
    calluri = QString(QLatin1String(ch));
}

void NemoSdkAdaptor::onCallInvited(const SDKCallInfo &callInfo)
{
    qDebug() << "onCallInvited: name=" << callInfo.remoteName.str() << ", uri=" << callInfo.remoteUri.str() << ", number=" << callInfo.remoteNumber.str();
    sigCallInvited(callInfo);
}

void NemoSdkAdaptor::onHowlingDetected(bool detected)
{
    qDebug() << "onHowlingDetected, detected=" << detected;
}

void NemoSdkAdaptor::onVideoStatusChanged(SDKVideoStatus status)
{
    qDebug() << "onVideoStatusChanged, status=" << status;
    sigVideoStateChanged(status);
}

void NemoSdkAdaptor::onCallStateChanged(const SDKCallStateInfo &callStateInfo)
{
    qDebug() << "onCallState:" << callStateInfo.callState << callStateInfo.sdkError.str()
             << callStateInfo.displayName.str() << callStateInfo.isOnhold
             << callStateInfo.onHoldCount << callStateInfo.waitingReason.str();
    sigConfStateChanged(callStateInfo);
}

void NemoSdkAdaptor::onNetworkStateChanged(bool connected)
{
    this->connected = connected;
    qDebug() << "onNetworkStateChanged:" << connected;
    log(QString("onNetworkStateChanged : %1").arg(connected));
    sigConnectionChanged(connected);
}

void NemoSdkAdaptor::onVideoStreamChanged(const XYArray<SDKVideoStreamInfo>& streams)
{
    qDebug() << "onVideoStreams:" << streams.length();

    std::vector<SDKVideoStreamInfo> streamVec;

    for (int i = 0; i < streams.length(); i++) {
        streamVec.push_back(streams[i]);
#if 0
        qDebug() << "onVideoStreams, dateSourceId : " << streams[i].videoSourceId.str() << ", state : " << streams[i].videoState << ", name : "
                 << QString::fromUtf8(streams[i].displayName.str())
                 << streams[i].oriWidth << streams[i].oriHeight
                 << streams[i].videoWidth << streams[i].videoHeight
                 << streams[i].extUserId.str();
#endif
    }
    sigStreamChanged(streamVec);
}

void NemoSdkAdaptor::onContentStateChanged(SDKContentState state)
{
    qDebug() << "onContentState:" << state;
    if(state == SDKContentState_Sending)
    {
        contentSharing = true;
    }
    else
    {
        contentSharing = false;
    }
    sigContentStateChanged(state);
}

void NemoSdkAdaptor::onNetworkIndicatorLevel(int level)
{
    qDebug() << "onNetworkIndicatorLevel:" << level;
    sigNetworkIndicatorLevel(level);
}

void NemoSdkAdaptor::onDeviceChanged(const SDKMediaDevType& devType)
{
    qDebug() << "onDeviceChanged:" << devType;
    sigDeviceChanged(devType);
}

void NemoSdkAdaptor::onMicMuted(bool muted)
{
    qDebug() << "onMicMuted:" << muted;
}

void NemoSdkAdaptor::onMicEnergyReported(int energy)
{
    emit sigMicEnergyReported(energy);
}

void NemoSdkAdaptor::onConfInfoChanged(const SDKConfInfo &info)
{
    log(QString("onConfInfoChanged, pcount : %1, chairman : %2").arg(info.participantCount).arg(info.chairManUrl.str()));
    qDebug() << "onConfInfoChanged:" << info.participantCount << ", chairman : " << info.chairManUrl.str() << ", as: " << info.actSpkUri.str();
    confEpNumChange(info.participantCount);
    sigConfMemberChanged(info.participantCount, info.contentPartCount > 0, QString::fromUtf8(info.chairManUrl.str()));
}

void NemoSdkAdaptor::onConfMgmtChanged(const SDKConfMgmtInfo & info)
{
    log(QString("onConfMgmtChanged, chairman : %1").arg(info.chirmanUri.str()));
    qDebug() << "onConfMgmtChanged, chairman : " << info.chirmanUri.str() << ", confMgmtUrl : " << info.confMgmtUrl.str();
    recordEnable = !info.disableRecord;
    sigConfMgmtChanged(info);
}

void NemoSdkAdaptor::onRosterChanged(const SDKRosterInfo &info)
{
    qDebug() << "onRosterChanged:" << info.totalEpNum;
    for (int i = 0; i < info.addParticipants.length(); i++) {
        qDebug() << "onRosterChanged add: " << info.addParticipants[i].callUri.str();
    }
    for (int i = 0; i < info.delParticipants.length(); i++) {
        qDebug() << "onRosterChanged del: " << info.delParticipants[i].str();
    }
    for (int i = 0; i < info.changeParticipants.length(); i++) {
        qDebug() << "onRosterChanged chg: " << info.changeParticipants[i].callUri.str();
    }
    log(QString("onRosterChanged, totalNum : %1, rosterType : %2").arg(info.totalEpNum).arg(info.rosterType));
    sigRosterChanged(info);
}


void NemoSdkAdaptor::onVideoFrameReady(const char* dataSourceId, const SDKVideoFrame& videoFrame)
{
    if (videoFrame.bufLen < 0 || videoFrame.bufLen > 3317760) {
        return;
    }
    //qDebug() << "onVideoFrameReady, dataSourceId : " << dataSourceId << ", rotation : " << videoFrame.rotation;
    //log(QString("onVideoFrameReady, sourceId : %1, width : %2, height : %3").arg(dataSourceId).arg(videoFrame.width).arg(videoFrame.height));
    if(videoFrame.buffer != nullptr && videoFrame.bufLen > 0)
    {
        SDKVideoFrameWrapper frameWrapper;
        frameWrapper.width = videoFrame.width;
        frameWrapper.bufLen = videoFrame.bufLen;
        frameWrapper.buffer = std::shared_ptr<uint8_t>(new (std::nothrow)uint8_t[videoFrame.bufLen], std::default_delete<uint8_t[]>());
        memcpy(frameWrapper.buffer.get(), videoFrame.buffer, videoFrame.bufLen);
        frameWrapper.height = videoFrame.height;
        frameWrapper.flipping = videoFrame.flipping;
        frameWrapper.rotation = videoFrame.rotation;
        frameWrapper.sarWidth = videoFrame.sarWidth;
        frameWrapper.sarHeight = videoFrame.sarHeight;
        sigVideoFrameReady(QString::fromUtf8(dataSourceId), frameWrapper);
    }
}

void NemoSdkAdaptor::onWhiteboardMessages(const XYArray<XYString>& msgs)
{
	const char*ch;
	int length = msgs.length();
	for (int i = 0; i < length; i++)
	{
		XYString jsonstr = msgs[i].str();
		ch = jsonstr.str();
		QString str = QString(QLatin1String(ch));
		qDebug() << "whiteboard(json)" + str;
		emit sigRecvWhiteboardJson(str);
	}
	if (calluri.length() > 2)
	{
		emit sigRecvWhiteboardCalluri(calluri);
	}
}

void NemoSdkAdaptor::onWhiteboardStateChanged(const SDKWhiteboardState & state)
{
	int i = state;
	qDebug() << "white state :" + QString::number(i);
    emit sigRecvWhiteboardFlag(i);
}

void NemoSdkAdaptor::NemoSdkAdaptor::onConfHandupCancelled()
{
    sigConfHandupCancelled();
}

void NemoSdkAdaptor::onGPUInfoResult(const XYArray<XYString> &gInfo)
{
    for(int i = 0; i < gInfo.length(); i++) {
        qDebug() << "onGPUInfoResult: " << gInfo[i].str();
    }
    _sdk->setEnableGPUs(gInfo);
}

void NemoSdkAdaptor::onAIFaceRecv(const SDKAIParam &info)
{
    //qDebug() << __FUNCTION__ << "isLocal: " << info.isLocal << "type: " << info.type;
}

void NemoSdkAdaptor::onAIFaceInfo(const XYString &data)
{
    //qDebug() << __FUNCTION__;
}

void NemoSdkAdaptor::onRecordStatusNotification(int callIndex, bool isStart, const XYString &uri, const XYString &callUri,
                                                const XYString &status)
{
    qDebug() << __func__ << isStart << uri.str();
}

void NemoSdkAdaptor::onRecordingStateChanged(SDKRecordingState recordState, const XYString &reason, bool isRecorded)
{
    qDebug() << __func__ << recordState << reason.str();
}

void NemoSdkAdaptor::onMeetingMuteQuery(const XYString &meetingId, const XYString &key)
{
    qDebug() << __func__ << meetingId.str() << key.str();
}

void NemoSdkAdaptor::onLoguploadResult(const SDKRestApiResult &result)
{
    qDebug() << __func__ << result.code.str() << result.body.str();
    sigLoguploadResponse(result);
}

void NemoSdkAdaptor::onConfHostChanged(const SDKConfHost &meetingHost)
{
    qDebug() << __func__ << meetingHost.meetingId.str();
    sigConfHostChanged(meetingHost.isHost);
}

void NemoSdkAdaptor::onInOutReminder(const XYArray<SDKInOutInfo> &info)
{
    for (int i = 0; i < info.length(); ++i) {
        qDebug() << __func__ << info[i].action.str() << info[i].extUserId.str() << info[i].displayName.str();
    }
}

void NemoSdkAdaptor::onConfCaptionChanged(const SDKConfCaption &notify)
{
    qDebug() << __func__ << notify.content.str();
    sigKConfCaptionChanged(notify);
}

void NemoSdkAdaptor::onSupportAiCaptionResult(bool isSupport)
{
    sigSupportAiCaptionResult(isSupport);
}

void NemoSdkAdaptor::onAiCaptionInfoChanged(const SDKAiCaptionInfo &info)
{
    sigAiCaptionInfoChanged(info);
}

void NemoSdkAdaptor::setVideoFlip(bool filp)
{
    _bVideoFilp = filp;
}

bool NemoSdkAdaptor::getVideoFlip() const
{
    return _bVideoFilp;
}
