﻿#ifndef XYSETTINGS_H
#define XYSETTINGS_H

#include <QDialog>
#include <QMouseEvent>
#include <QSettings>
#include <QButtonGroup>
#include <QSlider>
#include <QMutex>
#include <QStyledItemDelegate>
#include "meeting/OpenGLDisplay.h"
#include "NemoSdkAdaptor.h"
#include "meeting/FrameLayout.h"

#include "wayland/yuvdisplaylabel.h"

#pragma execution_character_set("utf-8")

const QString HTTP_PROXY_HOST = "http_proxy_host";
const QString HTTP_PROXY_PORT = "http_proxy_port";
const QString UDP_PROXY_HOST = "udp_proxy_host";
const QString UDP_PROXY_PORT = "udp_proxy_port";
const QString SOCK5_PROXY_HOST = "sock5_proxy_host";
const QString SOCK5_PROXY_PORT = "sock5_proxy_port";
const QString SOCK5_PROXY_NAME = "sock5_proxy_uname";
const QString SOCK5_PROXY_PWD = "sock5_proxy_pwd";

namespace Ui {
class XYSettings;
}

class XYSettings : public QDialog
{
    Q_OBJECT

public:
    explicit XYSettings(QDialog *parent = nullptr, bool incall = false);
    ~XYSettings();
    enum SettingType { COMMON, AUDIO, VIDEO, BEAUTY, FEEDBACK, PROXY };
    enum CheckType { SPEAKER, MIC };
    enum VideoResolution { R_180P, R_360P, R_720P, R_1080P };
    enum VideoFPS { FPS_15, FPS_30 };
    const QString DEFAULT_ID = "default";
    const QString DEFAULT_NAME = tr("系统默认");
    const QString MIC_ID_KEY = "micId";
    const QString SPK_ID_KEY = "spkId";
    const QString SPK_NAME_KEY = "spkName";
    const QString MIC_NAME_KEY = "micName";
    const QString CAMERA_ID_KEY = "cameraId";

public:
    void updateUi();
    void getAudioDevices(SDKMediaDevType type, int flag = -1);
    void getVideoDevices(int flag = -1);
    void timerCheckEvent(CheckType checkType, bool isStart);
    void onVideoFrameReady(const QString &dataSourceId, const SDKVideoFrameWrapper& videoFrame);
    void notifyDeviceToastTips();

protected:
    void showEvent(QShowEvent *event) override;
    void hideEvent(QHideEvent *event) override;
    void mouseMoveEvent (QMouseEvent *);
    void mousePressEvent (QMouseEvent *);
    void mouseReleaseEvent(QMouseEvent *);
    void initVirtualBg();
    void initBeauty();
    void initFilter();
    void initProxy();
    QString converImage2ipg(const QString& imgPath);

signals:
    void sigServerChanged() const;
    void sigDeviceChangedNotify(const QString& name);
    void sigProxyChange(std::map<ProxyType, ProxyNetAddress> proxyInfo);

public slots:
    void onSaveClicked();
    void onResetClicked();
    void onSliderVolumeInChanged(int value);
    void onSliderVolumeOutChanged(int value);
    void onSettingButtonsChecked(int id);
    void onAudioCheckGroupClicked(int id);
    void onSpeakerSelected(int idex);
    void onMicSelected(int idex);
    void onCameraSelected(int idx);
    void onVideoResolutionChanged(int index);
    void onVideoFrameChanged(int index);
    void onVideoSettingConfirmClicked();
    void onDeviceChanged(const SDKMediaDevType& devType);
    void onVirtualBgButtonsChecked(int id);
    void onBeautyButtonsChecked(int id);
    void onFilterButtonsChecked(int id);
    void onBeautySliderChanged(int value);
    void onBeautyTabClicked(int index);
    void onLoguploadResponse(const SDKRestApiResult& result);

private:
    Ui::XYSettings *ui;
    QPoint m_Position;
    bool isPressed;
    QSettings* configs;
    QString lastServerConfig; // 上次的server设置
    QString lastClientIdConfig; // 上次的tClientId设置

    QButtonGroup* toolBar;
    QTimer* speakerTimer; // 扬声器检测timer
    QTimer* micTimer;     // 麦克风检测timer
    bool startMicCheck = false;
    bool startSpeakerCheck = false;
    QWidget* openGL;
    QWidget* beautyOpenGL;

    // 视频发送帧率
    VideoResolution m_video_res = R_720P;
    VideoFPS m_video_fps = FPS_15;

    // 虚拟背景
    bool _needCloseCamera = true;
    FrameLayout* fl = nullptr;
    QSlider *beautySliders[BeautyStyle_Blush];
    QSlider *filterSliders[FilterStyle_Retro];
    QButtonGroup* _beautyBtGroup = nullptr;
    QButtonGroup* _filterBtGroup = nullptr;
    enum TabType {Tab_VirtualBg, Tab_Beauty, Tab_Filter};
    QSlider* createSlidbar(TabType type);
    int _beautyCheckId = 0;
    int _virtualBgCheckId = 0;
    int _filterCheckId = 0;
    QList<QString> _bgPath;
    struct FilterPath {
        QString name;
        QString path;
    };
    QList<FilterPath> _filterPath; // just for demo convenience
    const int COLUMN_VIRTUAL_BG = 4;
    const int COLUMN_FILTER = 6;
    bool bInCall;

    QMutex _lock;
    QTimer* _micDebounceTimer;
    QTimer* _spkDebounceTimer;
    QString _sysDefaultMicName;
    QString _sysDefaultSpkName;
    QString _toastMicName;
    QString _toastSpkName;
    int _lastSelectMicIndex = -1;
    int _lastSelectSpkIndex = -1;
};

#endif // XYSETTINGS_H
