﻿#include "FrameLayout.h"

#include <QWidget>

FrameLayout::FrameLayout(QWidget *parent, int margin, int hSpacing, int vSpacing)
{
    setContentsMargins(margin, margin, margin, margin);
}

FrameLayout::FrameLayout(int margin, int hSpacing, int vSpacing)
{
    setContentsMargins(margin, margin, margin, margin);
}

FrameLayout::~FrameLayout()
{
    QLayoutItem *item;
    while ((item = takeAt(0)))
        delete item;
}

void FrameLayout::addItem(QLayoutItem *item)
{
    itemList.append(item);
}

int FrameLayout::count() const
{
    return itemList.size();
}

QLayoutItem *FrameLayout::itemAt(int index) const
{
    return itemList.value(index);
}

void FrameLayout::setGeometry(const QRect &rect)
{
    QLayout::setGeometry(rect);
    int left, top, right, bottom;
    getContentsMargins(&left, &top, &right, &bottom);
    QRect effectiveRect = rect.adjusted(+left, +top, -right, -bottom);
    int x = effectiveRect.x();
    int y = effectiveRect.y();
    QLayoutItem *item;
    foreach (item, itemList) {
        QWidget *wid = item->widget();
        wid->setGeometry(x, y, effectiveRect.width() , effectiveRect.height());
    }
}

QSize FrameLayout::sizeHint() const
{
    return minimumSize();
}

QLayoutItem *FrameLayout::takeAt(int index)
{
    if (index >= 0 && index < itemList.size())
        return itemList.takeAt(index);
    else
        return 0;
}

QSize FrameLayout::minimumSize() const
{
    QSize size;
    QLayoutItem *item;
    foreach (item, itemList)
        size = size.expandedTo(item->minimumSize());

//    size += QSize(2 * margin(), 2 * margin());
    return size;
}

