﻿#include "MeetingLayout.h"

#include <iomanip>
#include <QWidget>
#include "VideoCell.h"

MeetingLayout::MeetingLayout(QWidget *parent, int margin, int hSpacing, int vSpacing) :
    layoutMode(Speaker)
{
    setContentsMargins(margin, margin, margin, margin);
}

MeetingLayout::~MeetingLayout()
{
    QLayoutItem *item;
    while ((item = takeAt(0)))
        delete item;
}

void MeetingLayout::addItem(QLayoutItem *item)
{
    itemList.append(item);
}

int MeetingLayout::horizontalSpacing() const
{
    if (m_hSpace >= 0) {
        return m_hSpace;
    }
}

int MeetingLayout::verticalSpacing() const
{
    if (m_vSpace >= 0) {
        return m_vSpace;
    }
}

QSize MeetingLayout::minimumSize() const
{
    return QSize(1024, 576); // 16:9
}

int MeetingLayout::count() const
{
    return itemList.size();
}

QLayoutItem *MeetingLayout::itemAt(int index) const
{
    return itemList.value(index);
}

void MeetingLayout::setGeometry(const QRect &rect)
{
    QLayout::setGeometry(rect);
    doLayout(rect);
}

QSize MeetingLayout::sizeHint() const
{
    return minimumSize();
}

QLayoutItem *MeetingLayout::takeAt(int index)
{
    if (index >= 0 && index < itemList.size())
        return itemList.takeAt(index);
    else
        return 0;
}

void MeetingLayout::doLayout(const QRect &rect)
{
    int left, top, right, bottom;
    getContentsMargins(&left, &top, &right, &bottom);
    QRect effectiveRect = rect.adjusted(+left, +top, -right, -bottom);
    int x = effectiveRect.x();
    int y = effectiveRect.y();
    int width = effectiveRect.width();
    int height = effectiveRect.height();

    if (width > height) {
        width = height * 16 / 9;
    } else {
        height = width * 9 / 16;
    }
    x = (effectiveRect.width() - width) / 2;
    y = (effectiveRect.height() - height) / 2;

    int smallCellWidth = width / 4;
    int smallCellHeight = smallCellWidth * 9 / 16;

    switch (layoutMode) {
    case Speaker: {
        if (itemList.size() == 1) {
            QWidget *wid = itemAt(0)->widget();
            wid->setGeometry(x, y, width, height);
            return;
        }
        int marginTop = height / 2 - smallCellHeight * 3 / 2;
        moveLocalCellByIndex(1);
        sortByActiveSpeaker();
        if (itemList.size() <= 4) {
            smallCellWidth = effectiveRect.width() / 4;
            smallCellHeight = smallCellWidth * 9 / 16;
            marginTop = effectiveRect.height() / 2 - smallCellHeight * 3 / 2;
            for (int i = 0; i < itemList.size(); ++i) {
                QWidget *wid = itemAt(i)->widget();
                if (i == 0) {
                    wid->setGeometry(effectiveRect.x(), effectiveRect.y() + marginTop, smallCellWidth * 3, smallCellHeight * 3);
                } else {
                    wid->setGeometry(smallCellWidth * 3 + effectiveRect.x(),
                                     effectiveRect.y() + marginTop + smallCellHeight * (i - 1),
                                     smallCellWidth,
                                     smallCellHeight);
                }
            }
        } else if (itemList.size() == 5) {
            for (int i = 0; i < itemList.size(); ++i) {
                QWidget *wid = itemAt(i)->widget();
                if (i == 0) {
                    wid->setGeometry(width / 2 - smallCellWidth * 3 / 2 + x, y, smallCellWidth * 3, smallCellHeight * 3);
                } else {
                    wid->setGeometry(x + smallCellWidth * (i - 1), height + y - smallCellHeight, smallCellWidth, smallCellHeight);
                }
            }
        } else if (itemList.size() > 5) {
            for (int i = 0; i < itemList.size(); i++) {
                QWidget *wid = itemAt(i)->widget();
                if (i == 0) {
                    wid->setGeometry(x, y, smallCellWidth * 3, smallCellHeight * 3);
                } else {
                    if (i > 3) {
                        wid->setGeometry(x + smallCellWidth * 3, height + y - smallCellHeight * (i - 3), smallCellWidth, smallCellHeight);
                    } else {
                        wid->setGeometry(x + smallCellWidth * (i - 1), height + y - smallCellHeight, smallCellWidth, smallCellHeight);
                    }
                }
            }
        }
        break;
    }
    case ContentShare:
    case Gallery:{
        int loopCondition = itemList.size();
        if (layoutMode == ContentShare) {
            loopCondition = itemList.size() > 4 ? 4 : itemList.size();
        } else {
            qDebug() << __func__ << "m_uiShowLocalWhenPageMode: " << m_uiShowLocalWhenPageMode << ", m_pageIndex: " << m_pageIndex;
            if (m_uiShowLocalWhenPageMode && (m_pageIndex == 1 || m_pageIndex == 0)) {
                moveLocalCellByIndex(0);
            } else {
                moveLocalCellByIndex(itemList.count() - 1);
                loopCondition = itemList.size() - 1;
            }
        }

        if (m_pageIndex > 1) {
            int dividex = (int)sqrt((double)LAYOUT_GALLERT_NUM);
            int dividey = dividex;
            if (LAYOUT_GALLERT_NUM ==  56) {
                dividex = 8;
                dividey = 7;
            }
            smallCellWidth = width / dividex;
            smallCellHeight = smallCellWidth * 9 / 16;
            for (int i = 0; i < loopCondition; i++) {
                QWidget *wid = itemAt(i)->widget();
                wid->setGeometry(x +  smallCellWidth * (i % dividex), y + smallCellHeight * (i / dividey), smallCellWidth, smallCellHeight);
            }
        } else {
            switch (loopCondition) {
            case 1:{
                itemAt(0)->widget()->setGeometry(x, y, smallCellWidth * 4, smallCellHeight * 4);
                break;
            }
            case 2: {
                smallCellWidth = effectiveRect.width() / 2;
                smallCellHeight = smallCellWidth * 9 / 16;
                int marginTop = (effectiveRect.height() - smallCellHeight) / 2;
                for (int i = 0; i < loopCondition; i++) {
                    QWidget *wid = itemAt(i)->widget();
                    wid->setGeometry(effectiveRect.x() + smallCellWidth * i, effectiveRect.y() + marginTop, smallCellWidth, smallCellHeight);
                }
                break;
            }
            case 3: {
                smallCellWidth = width / 2;
                smallCellHeight = smallCellWidth * 9 / 16;
                for (int i = 0; i < loopCondition; i++) {
                    QWidget *wid = itemAt(i)->widget();
                    if (i == 0) {
                        wid->setGeometry(x + smallCellWidth / 2, y, smallCellWidth, smallCellHeight);
                    } else {
                        wid->setGeometry(x + (i - 1) * smallCellWidth, y + smallCellHeight, smallCellWidth, smallCellHeight);
                    }
                }
                break;
            }
            case 4: {
                smallCellWidth = width / 2;
                smallCellHeight = smallCellWidth * 9 / 16;
                for (int i = 0; i < loopCondition; i++) {
                    QWidget *wid = itemAt(i)->widget();
                    wid->setGeometry(x + (i % 2) * smallCellWidth, y + (i / 2) * smallCellHeight, smallCellWidth, smallCellHeight);
                }
                break;
            }
            case 5:
            case 6: {
                smallCellWidth = width / 3;
                smallCellHeight = smallCellWidth * 9 / 16;
                int marginTop = (effectiveRect.height() - smallCellHeight * 2) / 2;
                for (int i = 0; i < loopCondition; i++) {
                    QWidget *wid = itemAt(i)->widget();
                    wid->setGeometry(x + (i % 3) * smallCellWidth, effectiveRect.y() + marginTop + (i / 3) * smallCellHeight, smallCellWidth, smallCellHeight);
                }
                break;
            }
            default: {
                int dividex = (int)sqrt((double)LAYOUT_GALLERT_NUM);
                int dividey = dividex;
                if (LAYOUT_GALLERT_NUM ==  56) {
                    dividex = 8;
                    dividey = 7;
                }
                smallCellWidth = width / dividex;
                smallCellHeight = smallCellWidth * 9 / 16;
                for (int i = 0; i < loopCondition; i++) {
                    QWidget *wid = itemAt(i)->widget();
                    wid->setGeometry(x +  smallCellWidth * (i % dividex), y + smallCellHeight * (i / dividey), smallCellWidth, smallCellHeight);
                }
                break;
            }
            }
        }
        break;
    }
    case Chairman:{
        moveLocalCellByIndex(1);
        int marginTop = height / 2 - smallCellHeight * 3 / 2;
        if (itemList.size() <= 3) {
            for (int i = 0; i < itemList.size(); ++i) {
                QWidget *wid = itemAt(i)->widget();
                if (i == 0) {
                    wid->setGeometry(x, y + marginTop, smallCellWidth * 3, smallCellHeight * 3);
                } else {
                    wid->setGeometry(smallCellWidth * 3 + x,
                                     y + marginTop + smallCellHeight * (i - 1),
                                     smallCellWidth,
                                     smallCellHeight);
                }
            }
        }
        break;
    }
    case Focus: {
        moveLocalCellByIndex(1);
        for (int i = 0; i < itemList.size(); ++i) {
            QWidget *wid = itemAt(i)->widget();
            if (i == 0) {
                wid->setGeometry(x, y, smallCellWidth * 4, smallCellHeight * 4);
            }
            if (i == 2) {
                wid->setGeometry(x, y + height - smallCellHeight, smallCellWidth, smallCellHeight);
                wid->raise();
                break;
            }
        }
        break;
    }
    case ContentOnly: {
        moveLocalCellByIndex(1);
        for (int i = 0; i < itemList.size(); ++i) {
            QWidget *wid = itemAt(i)->widget();
            if (i == 0) {
                wid->setGeometry(x, y, smallCellWidth * 4, smallCellHeight * 4);
                break;
            }
        }
        break;
    }
    case ContentAndAs: {
        moveLocalCellByIndex(1);
        smallCellWidth = effectiveRect.width() / 2;
        smallCellHeight = smallCellWidth * 9 / 16;
        int marginTop = (effectiveRect.height() - smallCellHeight) / 2;
        for (int i = 0; i < itemList.size(); ++i) {
            QWidget *wid = itemAt(i)->widget();
            if (i == 0) {
                wid->setGeometry(x, effectiveRect.y() + marginTop, smallCellWidth, smallCellHeight);
            }
            if (i == 2) {
                wid->setGeometry(x + smallCellWidth, effectiveRect.y() + marginTop, smallCellWidth, smallCellHeight);
                break;
            }
        }
        break;
    }
    case Locked: {
        //TODO: 双击锁定
        //for (int i = 0; i < itemList.size(); ++i) {
        //}
        break;
    }
    }
}

void MeetingLayout::moveLocalCellByIndex(int index)
{
    int count = itemList.count();
    if (index < count) {
        for (int i = 0; i < count; ++i) {
            VideoCell* cell = qobject_cast<VideoCell*>(itemList.at(i)->widget());
            if(cell != NULL) {
                QString sourceId(cell->getVideoInfo().videoSourceId.str());
                if(sourceId == "LocalPreviewID") {
                    itemList.move(i, index);
                }
            }
        }
    }
}

void MeetingLayout::sortByActiveSpeaker()
{
//    int contentIndex = -1;
//    int actSpkerIndex = -1;
//    for (int i = 0; i < itemList.count(); ++i) {
//        VideoCell* cell = qobject_cast<VideoCell*>(itemList.at(i)->widget());
//        if(cell != NULL) {
//            QString sourceId(cell->getVideoInfo().videoSourceId.str());
//            if (cell->getVideoInfo().isContent) {
//                itemList.move(i, 0);
//                contentIndex = i;
//            }

//            if(sourceId != "LocalPreviewID" && cell->getVideoInfo().isActiveSpeaker) {
//                itemList.move(i, 0);
//                actSpkerIndex = i;
//            }
//        }
//    }

//    if (contentIndex != -1 && actSpkerIndex != -1) {
//        if (contentIndex != actSpkerIndex) {
//            itemList.move(contentIndex, 0);
//            itemList.move(actSpkerIndex, itemList.size() - 1);
//        } else {
//            itemList.move(contentIndex, 0);
//        }
//    } else if (contentIndex != -1) {
//         itemList.move(contentIndex, 0);
//    } else if (actSpkerIndex != -1) {
//        itemList.move(actSpkerIndex, 0);
//    }
}

void MeetingLayout::setLayoutMode(MeetingLayout::LayoutMode layoutMode)
{
    this->layoutMode = layoutMode;
}

MeetingLayout::LayoutMode MeetingLayout::currentLayoutMode() const
{
    return layoutMode;
}

void MeetingLayout::setPageModeShowLocalPreview(bool uiShowLocalWhenPageMode)
{
    m_uiShowLocalWhenPageMode = uiShowLocalWhenPageMode;
}

bool MeetingLayout::isUIShowLocalWhenPageMode() const
{
    return m_uiShowLocalWhenPageMode;
}

void MeetingLayout::setPageModeIndex(int pageIndex)
{
    m_pageIndex = pageIndex;
}
