﻿#ifndef MEETINGLAYOUT_H
#define MEETINGLAYOUT_H

#include <QLayout>
#include <QRect>
#include <QStyle>
#include <math.h>

class MeetingLayout : public QLayout
{
public:
    static constexpr int LAYOUT_SPEAKER_NUM = 7; // demo speaker layout has 8 video cell include self so just need requre (8 - local) streams
    static constexpr int LAYOUT_GALLERT_NUM = 9;
	enum LayoutMode { Speaker, Gallery, Chairman, Focus, ContentOnly, ContentAndAs, ContentShare,Locked };
	explicit MeetingLayout(QWidget *parent = nullptr, int margin = -1, int hSpacing = -1, int vSpacing = -1);
	~MeetingLayout();

	void addItem(QLayoutItem *item) override;
	int horizontalSpacing() const;
	int verticalSpacing() const;
	QSize minimumSize() const override;
	int count() const override;
	QLayoutItem *itemAt(int index) const override;
	void setGeometry(const QRect &rect) override;
	QSize sizeHint() const override;
	QLayoutItem *takeAt(int index) override;

    void setLayoutMode(LayoutMode layoutMode);
	LayoutMode currentLayoutMode() const;

    void setPageModeShowLocalPreview(bool isShowLocalWhenPageMode);
    bool isUIShowLocalWhenPageMode() const;
    void setPageModeIndex(int pageIndex);

private:
	void doLayout(const QRect &rect);
    void moveLocalCellByIndex(int index);
    void sortByActiveSpeaker();

private:
	QList<QLayoutItem *> itemList;
    LayoutMode layoutMode = Speaker;
	int m_hSpace;
    int m_vSpace;
    bool m_uiShowLocalWhenPageMode = true;
    int m_pageIndex = 0;
};

#endif // MEETINGLAYOUT_H
