﻿#include "MeetingNotifyWidget.h"
#include "ui_meetingnotifywidget.h"

#include <QFile>
#include "NemoSdkAdaptor.h"
#include <QTimer>

//#include "src/relevance/uidefine.h"
MeetingNotifyWidget::MeetingNotifyWidget(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::MeetingNotifyWidget),
    _displayRule(CC_SHOW_NO)
{
    ui->setupUi(this);

    QFile file(":/stylesheet/style_meeting_notification.qss");
    if (file.open(QFile::ReadOnly)) {
        this->setStyleSheet(tr(file.readAll()));
        file.close();
    }
    ui->label_zimu->hide();      // 字幕消息
    ui->label_chairman->hide();  // 主会场
    ui->label_net_tips->hide();  // 网络状况
    ui->lable_outin->hide();     // 出入会通知
    ui->bt_outin_hide->hide();   // 隐藏
    ui->label_cc_src->hide();    // 同传字幕
    ui->label_cc_transalte->hide();

    connect(ui->bt_next_page, &QToolButton::clicked, this, [=]() {
        sigNextPageClicked();
    });
    connect(ui->bt_pre_page, &QToolButton::clicked, this, [=]() {
        sigPrePageClicked();
    });

    _timer = new QTimer(this);
    connect(_timer, &QTimer::timeout, this, [=]() {
        ui->label_cc_src->setVisible(false);
        ui->label_cc_transalte->setVisible(false);
    });

    ui->label_cc_src->setWordWrap(true);
    ui->label_cc_transalte->setWordWrap(true);

    qRegisterMetaType<SDKConfCaption>("SDKConfCaption");
    connect(NemoSdkAdaptor::getInstance(), &NemoSdkAdaptor::sigKConfCaptionChanged, this, [=](const SDKConfCaption& notify) {
        onNotificationChanged(Zimu, !QString(notify.content.str()).isEmpty() && QString(notify.action.str()) != "cancel");
        if (!QString(notify.content.str()).isEmpty() && QString(notify.action.str()) != "cancel") {
            showZimu(notify.content.str(), notify.backgroundRGB.str(), notify.fontRGB.str());
        }
    });
    qRegisterMetaType<SDKAiCaptionInfo>("SDKAiCaptionInfo");
    connect(NemoSdkAdaptor::getInstance(), &NemoSdkAdaptor::sigAiCaptionInfoChanged, this, [=](const SDKAiCaptionInfo& info) {
        _timer->stop();
        qDebug() << __func__ << info.src.str();
        ui->label_cc_src->setVisible(_displayRule != CC_SHOW_NO);
        ui->label_cc_transalte->setVisible(_displayRule == CC_SHOW_BOTH);

        QString captionSrc =  QString(info.src.str());
        QString captionTarget =  QString(info.target.str());
        QString srcLan(info.srcLang.str());
        QString targetLan(info.targetLang.str());

        if (_displayRule == CC_SHOW_ENGLISH) {
            if (targetLan == "en") {
                if (!captionTarget.isEmpty()) {
                    captionTarget.prepend(QString(info.dn.str()) + ": ");
                    ui->label_cc_src->setText(captionTarget);
                }
            } else {
                if (!captionSrc.isEmpty()) {
                    ui->label_cc_src->setText(captionSrc);
                }
            }
        } else if (_displayRule == CC_SHOW_CHINESE) {
            if (targetLan == "zh") {
                captionTarget.prepend(QString(info.dn.str()) + ": ");
                if (!captionTarget.isEmpty()) {
                    ui->label_cc_src->setText(captionTarget);
                }
            } else {
                if (!captionSrc.isEmpty()) {
                    ui->label_cc_src->setText(captionSrc);
                }
            }
        } else if (_displayRule == CC_SHOW_NO) {
            // ignore
        } else if (_displayRule == CC_SHOW_BOTH) {
            captionSrc.prepend(QString(info.dn.str()) + ": ");
            if (!captionSrc.isEmpty()) {
                ui->label_cc_src->setText(captionSrc);
            }
            if (!captionTarget.isEmpty()) {
                ui->label_cc_transalte->setText(captionTarget);
            }
        }

        if (!_timer->isActive()) {
            _timer->start(5000);
        }
    });

    // toast
    QWidget* pWid = new QWidget(this);
    QVBoxLayout* outLayout = new QVBoxLayout(this);
    outLayout->setContentsMargins(0, 0, 0, 0);
    _toastContainer = new QWidget(pWid);
    outLayout->addWidget(_toastContainer);
    outLayout->addStretch();
    pWid->setLayout(outLayout);
    _vBoxLayout = new QVBoxLayout(this);
    _vBoxLayout->setContentsMargins(0, 0, 0, 0);

    _toastContainer->setLayout(_vBoxLayout);
    pWid->setGeometry(this->parentWidget()->width() / 4, ui->label_zimu->y() + ui->label_zimu->height(),
                              this->parentWidget()->width() / 2, this->parentWidget()->height() / 2);
    _toastContainer->show();

    ui->bt_pre_page->setVisible(false);
    ui->bt_next_page->setVisible(false);
}

MeetingNotifyWidget::~MeetingNotifyWidget()
{
    delete ui;
    delete _timer;
}

void MeetingNotifyWidget::showZimu(const QString &content, const QString &color, const QString &fontColor,
                                   MeetingNotifyWidget::Position position, bool isScroll)
{
    ui->label_zimu->setText(content);
    ui->label_zimu->setStyleSheet(QString("QLabel{background-color: %1;font: 18pt \"Microsoft YaHei\";color: %2}")
                                  .arg(color).arg(fontColor));
    // TODO: position & isScroll
    ui->label_zimu->setText(content);
}

void MeetingNotifyWidget::onNotificationChanged(NotifyMode mode, bool visiable, const QString& tips)
{
    switch (mode) {
    case Zimu:
        if (ui->label_zimu->isVisible() != visiable)
            ui->label_zimu->setVisible(visiable);      // 字幕消息
        break;
    case PrePage:
        if (ui->bt_pre_page->isVisible() != visiable)
            ui->bt_pre_page->setVisible(visiable);     // 上一页
        break;
    case NextPage:
        if (ui->bt_next_page->isVisible() != visiable)
            ui->bt_next_page->setVisible(visiable);    // 下一页
        break;
    case OutIn:
        if (ui->bt_outin_hide->isVisible() != visiable) {
            ui->lable_outin->setVisible(visiable);     // 出入会通知
            ui->bt_outin_hide->setVisible(visiable);   // 隐藏
        }
        break;
    case Chairman:
        if (ui->label_chairman->isVisible() != visiable)
            ui->label_chairman->setVisible(visiable);  // 主会场
        break;
    case VideoState:
        if (ui->label_net_tips->isVisible() != visiable) {
            if (!tips.isEmpty()) {
                ui->label_net_tips->setText(tips);
            }
            ui->label_net_tips->setVisible(visiable);
        }
        break;
    case CCInfo:
        if (ui->label_cc_src->isVisible() != visiable) {
            if (!tips.isEmpty()) {
                ui->label_cc_src->setText(tips);
            }
            ui->label_net_tips->setVisible(visiable);
        }
        break;
    case Toast:
        QPushButton* bt = new QPushButton(this);

        bt->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Fixed);
        QString message = tips;
        QFontMetrics elideFont(bt->font());
        if (elideFont.width(message) > this->parentWidget()->width() / 2) {
            message = QFontMetrics(bt->font()).elidedText(message, Qt::ElideRight, this->parentWidget()->width() / 2 - 10);
        }
        bt->setText(message);
        bt->setCheckable(false);
        _vBoxLayout->insertWidget(0, bt);
        QTimer::singleShot(3000, this, [=]() {
            _vBoxLayout->removeWidget(bt);
            bt->deleteLater();
        });
        break;
    }
}

void MeetingNotifyWidget::showOut(const QString &outName)
{
    ui->lable_outin->setText(outName);
}

void MeetingNotifyWidget::showIn(const QString &inName)
{
    ui->lable_outin->setText(inName);
}

void MeetingNotifyWidget::setCcDisplayRule(MeetingNotifyWidget::CCDisplayRule display)
{
    _displayRule = display;
}

void MeetingNotifyWidget::onPreButton(bool flag)
{
    if (flag)
        ui->bt_pre_page->show();
    else
        ui->bt_pre_page->hide();
    raise();
}
void MeetingNotifyWidget::onNextButton(bool flag)
{
    if (flag)
        ui->bt_next_page->show();
    else
        ui->bt_next_page->hide();
    raise();
}
