﻿#include "MenuButton.h"
#include "ui_menubutton.h"

#pragma execution_character_set("utf-8")

MenuButton::MenuButton(QString text, QString normalIcon, QString hoverIcon, QWidget *parent) :
    QWidget(parent),
    hoverIcon(hoverIcon),
    normalIcon(normalIcon),
    _btDisable(false)
{
    setFixedSize(60, 60);
    frameLayout = new FrameLayout(this, 0 ,0 ,0);
    label = new QLabel(this);
    label->setContentsMargins(0, 6, 6, 0);
    label->setAlignment(Qt::AlignRight | Qt::AlignTop);
    label->setStyleSheet("QLabel{border:0px;color: white;font: 6pt \"Microsoft YaHei\";}");
    menuButton = new QToolButton(this);
    menuButton->setText(text);
    menuButton->setIcon(QIcon(normalIcon));
	menuButton->setIconSize(menuButton->rect().size());
    menuButton->setToolButtonStyle(Qt::ToolButtonTextUnderIcon);
    menuButton->setStyleSheet("QToolButton{border:0px;color: white;padding-top: 8px;padding-bottom: 8px;font: 6pt \"Microsoft YaHei\";}");
    frameLayout->addWidget(label);
    frameLayout->addWidget(menuButton);
    setLayout(frameLayout);

    label->hide();
    connect(menuButton, &QToolButton::clicked, this, &MenuButton::onClicked);
    setCursor(QCursor(Qt::PointingHandCursor));
}

MenuButton::~MenuButton()
{
    delete frameLayout;
    delete label;
    delete menuButton;
}

void MenuButton::leaveEvent(QEvent *) {
    menuButton->setStyleSheet(QString("QToolButton{border:0px;color: %1;padding-top: 8px;padding-bottom: 8px;font: 6pt \"Microsoft YaHei\";}")
                              .arg(_btDisable ? "gray" : "white"));
    menuButton->setIcon(QIcon(normalIcon));
    //    ui->tv_title->setText(text);
}

void MenuButton::enterEvent(QEvent *) {
    QString textColor = flag ? "#3876FF" : "#ff6666";
    menuButton->setStyleSheet(QString("QToolButton{border:0px;color: %1;padding-top: 8px;padding-bottom: 8px;font: 6pt \"Microsoft YaHei\";}")
                              .arg(_btDisable ? "gray" : textColor));
    menuButton->setIcon(QIcon(hoverIcon));
    //    ui->tv_title->setText(text);
}

void MenuButton::onClicked() {
    emit clicked();
}

void MenuButton::setChecked(QString text, QString normalIcon, QString hoverIcon) {
    this->flag = !flag;
    this->text = text;
    menuButton->setText(text);
    this->hoverIcon = hoverIcon;
    this->normalIcon = normalIcon;
    QString textColor = flag ? "#3876FF" : "#ff6666";
    menuButton->setStyleSheet(QString("QToolButton{border:0px;color: %1;padding-top: 8px;padding-bottom: 8px;font: 6pt \"Microsoft YaHei\";}")
                              .arg(_btDisable ? "gray" : "white"));
    menuButton->setIcon(QIcon(hoverIcon));
}

void MenuButton::setChecked(QString text, QString normalIcon, QString hoverIcon, bool showBuleColor) {
	this->flag = flag;
	this->text = text;
	menuButton->setText(text);
	this->hoverIcon = hoverIcon;
	this->normalIcon = normalIcon;
	QString textColor = flag ? "#3876FF" : "#ff6666";
	menuButton->setStyleSheet(QString("QToolButton{border:0px;color: %1;padding-top: 8px;padding-bottom: 8px;font: 6pt \"Microsoft YaHei\";}")
		.arg(_btDisable ? "gray" : "white"));
	menuButton->setIcon(QIcon(hoverIcon));
}

void MenuButton::resetState(QString text, QString normalIcon, QString hoverIcon, bool disable)
{
    _btDisable = disable;
    this->flag = true;
    this->text = text;
    menuButton->setText(text);
    this->hoverIcon = hoverIcon;
    this->normalIcon = normalIcon;
    menuButton->setStyleSheet(QString("QToolButton{border:0px;color: %1;padding-top: 8px;padding-bottom: 8px;font: 6pt \"Microsoft YaHei\";}")
                              .arg(_btDisable ? "gray" : "white"));
    menuButton->setIcon(QIcon(normalIcon));
}

void MenuButton::onWindowStateChange(Qt::WindowState state)
{
    if(state == Qt::WindowFullScreen) {
        resetState(tr("退出全屏"), ":/res/meeting/quit_full_screen_normal.png", ":/res/meeting/quit_full_screen_hover.png");
    } else {
        resetState(tr("全屏"), ":/res/meeting/full_screen_normal.png", ":/res/meeting/full_screen_hover.png");
    }
}

void MenuButton::showLabel(int num)
{
    label->show();
    label->setText(QString::number(num));
}
