﻿#ifndef OPENGLDISPLAY_H
#define OPENGLDISPLAY_H

#include <QOpenGLWidget>
#include <QOpenGLBuffer>
#include <QOpenGLFunctions>
#include <QScopedPointer>
#include <QException>
#include <memory>

class OpenGLDisplay : public QOpenGLWidget, public QOpenGLFunctions
{
public:
    explicit OpenGLDisplay(QWidget* parent = nullptr);
    ~OpenGLDisplay();

    void initBuffer(int bsize);
    void requestRender(const std::shared_ptr<uint8_t>& data, int length, int width, int height, int sarWidth, int sarHieght, int rotation, bool flip);
    void clearDraw();

protected:
    void initializeGL() override;
    void resizeGL(int w, int h) override;
    void paintGL() override;
    void showEvent(QShowEvent *event) override;

private:
    struct OpenGLDisplayImpl;
    QScopedPointer<OpenGLDisplayImpl> impl;
    QOpenGLBuffer vbo;
    GLfloat _vertices[8];
    GLfloat _coord[8];
    unsigned int _viewWidth;
    unsigned int _viewHeight;
    static const GLfloat _defaultVertices[];
    static const GLfloat _defaultCoord[];
    static const GLfloat _coord_90[];
    static const GLfloat _coord_180[];
    static const GLfloat _coord_270[];

    static const GLfloat _defaultCoord_mirror[];
    static const GLfloat _coord_90_mirror[];
    static const GLfloat _coord_180_mirror[];
    static const GLfloat _coord_270_mirror[];
};

#endif // OPENGLDISPLAY_H
