﻿#include "CellState.h"
#include "VideoCell.h"

#include <QHBoxLayout>

#pragma execution_character_set("utf-8")

//#define DISABLE_OPENGL

VideoCell::VideoCell(QWidget *parent) :
    QWidget(parent)
{
#ifdef DISABLE_OPENGL
	display = new YUVDisplayLabel(this);
#else
	display = new OpenGLDisplay(this);
#endif

    displayName = new QLabel(display);
    displayName->setText("");
    displayName->setNum(30);
    displayName->setStyleSheet("QLabel{background-color:#2A2E33;padding:3px;border-radius:3px;font: 12px \"Microsoft YaHei\"; color:white;}");
    displayName->adjustSize();

    state = new CellState(this);
    state->setAttribute(Qt::WA_StyledBackground,true);

    QVBoxLayout* stateLayout = new QVBoxLayout(this);
    stateLayout->addWidget(state);
    stateLayout->setContentsMargins(0,0,0,0);

    display->setLayout(stateLayout);

    QVBoxLayout* displayLayout = new QVBoxLayout(this);
    displayLayout->setContentsMargins(0,0,0,0);
    displayLayout->addWidget(display);

    audioState = new QLabel(display);
    audioState->setStyleSheet("QLabel{image:url(:/res/meeting/mic_mute_on_video.png);}");
    audioState->adjustSize();
    audioState->resize(30, 30);
    audioState->move(5, 5);
    setLayout(displayLayout);
}

VideoCell::~VideoCell()
{
    delete display;
}

void VideoCell::resizeEvent(QResizeEvent *event)
{
    displayName->move(5, this->height() - displayName->height() - 5);
}

void VideoCell::setVidoeInfo(SDKVideoStreamInfo info, bool showSmallState)
{
    videoInfo = info;
    audioState->setVisible(showSmallState);
    displayName->setVisible(showSmallState);

    QFontMetrics fontWidth(displayName->font());
    QString elideNote = fontWidth.elidedText(info.displayName.str(), Qt::ElideRight, 180);
    displayName->setText(elideNote);

    state->setDisplayName(showSmallState ? info.displayName.str() : "");
    displayName->adjustSize();
	if(showSmallState==true)
    audioState->setVisible(info.audioMute);

    switch(info.videoState)
    {
    case SDKVideoState_VideoMute:
        state->setVisible(true);
        state->setVideoState(showSmallState ? tr("视频暂停") : "");
        break;
    case SDKVideoState_Requesting:
        state->setVisible(true);
        state->setVideoState(tr("视频请求中..."));
        break;
    case SDKVideoState_AudioOnly:
        state->setVisible(true);
        state->setVideoState(tr("语音模式"));
        break;
    case SDKVideoState_NoDecoder:
        state->setVisible(true);
        state->setVideoState(tr("视频暂停"));
        break;
    case SDKVideoState_NoBandwidth:
        state->setVisible(true);
        state->setVideoState(tr("网络不给力，暂时无法接收视频"));
        break;
    case SDKVideoState_Idle:
        break;
    case SDKVideoState_Telephone:
        break;
    case SDKVideoState_Received:
        state->setVisible(false);
        break;
    case SDKVideoState_Observing:
        break;
    default:
        break;
    }
}

SDKVideoStreamInfo VideoCell::getVideoInfo() const
{
    return videoInfo;
}

QWidget* VideoCell::getDisplay()const
{
    return display;
}

void VideoCell::setLocalAudioMute(bool mute) {
    audioState->setVisible(mute);
}

void VideoCell::requestRender(const std::shared_ptr<uint8_t>& data, size_t length, int width, int height, int sarWidth, int sarHeight, int rotation, bool flip)
{
#ifdef DISABLE_OPENGL
	static_cast<YUVDisplayLabel*>(display)->initBuffer(width * height * 3 / 2);
	static_cast<YUVDisplayLabel*>(display)->requestRender(data, length, width, height, sarWidth, sarHeight, rotation, flip);
#else
	static_cast<OpenGLDisplay*>(display)->initBuffer(width * height * 3 / 2);
	static_cast<OpenGLDisplay*>(display)->requestRender(data, length, width, height, sarWidth, sarHeight, rotation, flip);
#endif
}
