﻿#include "ccdialog.h"
#include "ui_ccdialog.h"

#include <QDebug>
#include <QLocale>

#pragma execution_character_set("utf-8")

CcDialog::CcDialog(QWidget* baseWidget, QWidget *parent) :
    QDialog(parent),
    ui(new Ui::CcDialog),
    _baseWidget(baseWidget)
{
    ui->setupUi(this);
    this->setWindowFlags(Qt::FramelessWindowHint);
    this->setModal(true);
    this->setAttribute(Qt::WA_DeleteOnClose);
    if (_baseWidget != nullptr) {
        setGeometry(_baseWidget->geometry().x(), _baseWidget->parentWidget()->parentWidget()->y()  - this ->height(),
                    this->width(), this->height());
    }
    ui->spk_auto->setChecked(true);

    QLocale locale;
    if (locale.language() == QLocale::Language::Chinese) {
        ui->cc_en->setChecked(true);
        ui->spk_auto->setText(tr("自动(简体中文)"));
        _srcLaguage = Speak_Chinese;
    } else {
        ui->cc_cn->setChecked(true);
        ui->spk_auto->setText(tr("Auto(English)"));
        _srcLaguage = Speak_English;
    }

    _groupSpk = new QButtonGroup(this);
    _groupSpk->addButton(ui->spk_cn, Speak_Chinese);
    _groupSpk->addButton(ui->spk_en, Speak_English);
    _groupSpk->addButton(ui->spk_auto, Speak_Auto);
    _groupCc = new QButtonGroup(this);
    _groupCc->addButton(ui->cc_cn, CC_Language_Chinese);
    _groupCc->addButton(ui->cc_en, CC_Language_English);
//    connect(_groupSpk, QOverload<int>::of(&QButtonGroup::buttonClicked), this, [=](int id) {
//        switch (id) {
//        case Speak_Auto:
//            sigLanguageSelected(_srcLaguage, CC_Language_Ignore, ui->rb_show_both->isChecked());
//            break;
//        case Speak_Chinese:
//            sigLanguageSelected(Speak_Chinese, CC_Language_Ignore, ui->rb_show_both->isChecked());
//            break;
//        case Speak_English:
//            sigLanguageSelected(Speak_Chinese, CC_Language_Ignore, ui->rb_show_both->isChecked());
//            break;
//        }
//    });
//    connect(_groupCc, QOverload<int>::of(&QButtonGroup::buttonClicked), this, [=](int id) {
//        switch (id) {
//        case CC_Language_Chinese:
//            sigLanguageSelected(Speak_Ignore, CC_Language_Chinese, ui->rb_show_both->isChecked());
//            break;
//        case CC_Language_English:
//            sigLanguageSelected(Speak_Ignore, CC_Language_English, ui->rb_show_both->isChecked());
//            break;
//        }
//    });
    _groupCc->setExclusive(false);

    connect(ui->cc_cn, &QRadioButton::clicked, this, [=]() {
        ui->cc_cn->setChecked(ui->cc_cn->isChecked());
        if (ui->cc_cn->isChecked() && ui->cc_en->isChecked()) {
            ui->cc_en->setChecked(false);
        }
    });

    connect(ui->cc_en, &QRadioButton::clicked, this, [=]() {
        ui->cc_en->setChecked(ui->cc_en->isChecked());
        if (ui->cc_en->isChecked() && ui->cc_cn->isChecked()) {
            ui->cc_cn->setChecked(false);
        }
    });
}

CcDialog::~CcDialog()
{
    delete ui;
}

void CcDialog::hideEvent(QHideEvent *event)
{
    switch (_groupSpk->checkedId()) {
    case Speak_Auto:  {
        QLocale locale;
        if (locale.language() == QLocale::Language::Chinese) {
            _srcLaguage = Speak_Chinese;
        } else {
            _srcLaguage = Speak_English;
        }
        break;
    }
    case Speak_Chinese:
        _srcLaguage = Speak_Chinese;
        break;
    case Speak_English:
        _srcLaguage = Speak_English;
        break;
    }
    if (ui->rb_show_both->isChecked()) {
        sigLanguageSelected(_srcLaguage, CC_Language_BOTH);
    } else if (_groupCc->checkedId() == CC_Language_Chinese) {
        sigLanguageSelected(_srcLaguage, CC_Language_Chinese);
    } else if (_groupCc->checkedId() == CC_Language_English) {
        sigLanguageSelected(_srcLaguage, CC_Language_English);
    } else {
        sigLanguageSelected(_srcLaguage, CC_Language_None);
    }
}
