﻿#ifndef WAYLANDCAPTURE_H
#define WAYLANDCAPTURE_H

#ifdef WAYLAND

#include <QMainWindow>
#include<QLabel>
#include <memory>

#include "connection_thread.h"
#include "event_queue.h"
#include "registry.h"
#include "output.h"
#include "remote_access.h"
// Qt
#include <QGuiApplication>
#include <QDebug>
#include <QThread>
#include <QTimer>
#include <QTime>
#include <QImage>
#include <QStringList>
// system
#include <fcntl.h>
#include <unistd.h>
#include <sys/mman.h>
//#include "sendimage.h"
#include <QPushButton>
#include <QLocalServer>
#include <QLocalSocket>

using namespace KWayland::Client;

struct remote_buffer {
    RemoteBuffer* remoteBuffer = nullptr;
    int32_t fd = 0;
    uint32_t x;
    uint32_t y;
    uint32_t width;
    uint32_t height;
    uint32_t stride;
    uint32_t format;
    bool mapped = false;
    unsigned char *mapped_data;
};

class WaylandCapture : public QObject
{
    Q_OBJECT
public:
    explicit WaylandCapture(QObject* parent = nullptr);
    virtual ~WaylandCapture();

    void processBuffer(const Output* m_output,  RemoteBuffer* m_remoteBuffer);

    void startContentCapture();
    void stopContentCapture();
    void pauseContentCapture();
    void restartContentCapture();

private:
    void setupRegistry(Registry* registry);
    QThread* m_connectionThread = nullptr;
    ConnectionThread* m_connectionThreadObject;
    EventQueue* m_eventQueue = nullptr;
    Compositor* m_compositor = nullptr;
    RemoteAccessManager* m_remoteAccessManager = nullptr;
    QList<Output*> m_outputList;
    Output* m_bufferReadyOutput = nullptr;
    Output* m_output0 = nullptr;
    Output* m_output1 = nullptr;
    //    SendImage *m_SendImage;
    QPushButton* sendData;
    RemoteBuffer* m_rbuf;
    Registry* registry;

    size_t _bufLen;
    int _width = 0;
    int _height = 0;
    int _stride = 0;
    uint64_t _captureFrame = 0;
    //    unsigned char *_rgbBuffer = nullptr;
    std::shared_ptr<uint8_t> _rgbBuffer;

    bool _isFirstStart = false;
    bool _isStopGrapScreen = true;
    bool _isPause = false;

    remote_buffer m_buffer;
};

#endif


#endif // WAYLANDCAPTURE_H
